/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package Abstrakt.ADL;

import java.io.Serializable;
import Abstrakt.IIterator;

/**
 * Abstraktní datová struktura
 * cyklicky zřetězený seznam bez hlavy
 * @author Administrator
 * @param <objType> vstupní datový typ (struktura, která bude ukládána)
 */
public abstract class AbstrDoubleList<objType> implements IAbstrDoubleList<objType>, Serializable {

    private IPrvek<objType> begin = new Prvek<objType>();   // začátek seznamu
    private IPrvek<objType> end = new Prvek<objType>();     // konec seznamu
    private IPrvek<objType> actual = new Prvek<objType>();  // aktuální pozice
    private boolean pohybAktualnimPriVkladani = true;      // bude pohybovat aktuálním prvkem při vkládání prvku, tru = pohybuje

    /**
     * bezparametrický konstruktor, který nastaví všechny proměnné na null
     */
    public AbstrDoubleList() {
        this.begin = null;
        this.end = null;
        this.actual = null;
    }

    /**
     * Iterátor ADS, slouží pro procházení celé struktury po jednom prvku
     * @param <objIt> struktura, ve které budou vraceny data z ADL iterátoru
     */
    private class Iterator implements IIterator, Serializable {

        private IPrvek<objType> position;
        private boolean firstPass = true;
        private TypIteratoru typIteratoru;

        public Iterator(TypIteratoru typIteratoru) {
            this.typIteratoru = typIteratoru;
        }

        /**
         * Je již "position" datovým typem, nebo je "null"?
         * @return true = již existuje instance atributu position
         */
        private boolean instanceofPosition() {
            return this.position instanceof IPrvek;
        }

        /**
         * Vrací zda-li má ADL další prvek v seznamu (pokud by byl acyklický)
         *
         * Přepsaný a opravený hasNext_original
         * 
         * Postupně kontroluje jestli ADL existuje, jeslti má již instanci 
         * proměnná position, zda-li je to, či není první průchod iterátorem,
         * a zda-li se iterátor nenachází na pomyslném konci seznamu.
         *
         * @return true = má další prvek
         */
        public boolean hasNext() {
            if (jePrazdny()) {
                return false;
            } else if (!instanceofPosition()) {
                if (typIteratoru == TypIteratoru.OD_PREDU) {
                    position = (IPrvek<objType>) begin;
                } else {
                    position = (IPrvek<objType>) end;
                }
            } else if (jePosledniVSeznamu()) {
                if (firstPass) {
                    firstPass = false;
                    return true;
                }
                return false;
            } else if (jePrvni(this.position) && (typIteratoru == TypIteratoru.OD_PREDU)) { //.getNext().getPrev(), předtím bez toho nefungoval :D
                if (firstPass) {
                    firstPass = false;
                    return true;
                }
                return false;
            } else if (jePosledni(this.position) && (typIteratoru == TypIteratoru.OD_ZADU)) { //.getNext().getPrev(), předtím bez toho nefungoval :D
                if (firstPass) {
                    firstPass = false;
                    return true;
                }
                return false;
            }
            /*if (!firstPass) {
            return false;
            }*/

            /*if (jePrvni(this.position)) { //.getNext().getPrev(), předtím bez toho nefungoval :D
            return false;
            }*/
            firstPass = false;
            return true;
        }

        /**
         * Posouvá iterátor na další prvek v seznamu a zároveň vrací aktuální
         * datovou strukturu
         * @return obsah datové struktury aktuálního prvku
         */
        public objType next() {
            if (jePrazdny()) {
                return null;
            }
            if (!instanceofPosition()) {
                if (typIteratoru == TypIteratoru.OD_PREDU) {
                    position = (IPrvek<objType>) begin;
                } else {
                    position = (IPrvek<objType>) end;
                }
            }

            IPrvek<objType> iPrvek_data = this.position;
            /*if (this.position.getData() == null) {
            this.position = (IPrvek<objType>) begin;
            }*/
            if (jePosledniVSeznamu()) {

                if (typIteratoru == TypIteratoru.OD_PREDU) {
                    this.position = (IPrvek<objType>) begin;
                } else {
                    this.position = (IPrvek<objType>) end;
                }
            }
            actual = this.position; // kvůli moveToFront a najdi
            if (typIteratoru == TypIteratoru.OD_PREDU) {
                this.position = this.position.getNext();
            } else {
                this.position = this.position.getPrev();
            }
            return iPrvek_data.getData();
        }

        public objType dalsi() {
            return next();
        }

        public boolean jeDalsi() {
            return hasNext();
        }
    }

    /**
     * Zjistí zda-li je ADL prázdný
     * @return true = je prázdný, false = není prázdný
     */
    public boolean jePrazdny() {
        if (this.actual == null) {
            return true;
        }/* else {
        if(end.isEmpty()) {
        return true;
        }
        return false;
        }*/
        return false;
    }

    /**
     * Zrušení celého seznamu
     *
     * Postupně prochází celý seznam a ruší jednotlivé prvky, následně
     * zruší i aktuální, začátek a konce seznamu, čímž způsobí,
     * že Garbage collector dealokuje celou ADL
     */
    public void Zrus() {
        if (jePrazdny()) {
            return;
        }
        if (this.actual.getNext() == null) {
            this.actual = null;
            this.begin = null;
            this.end = null;
            return;
        }
        while (!jePosledniVSeznamu()) {
            odeberPosledni();
        }
        odeberPosledniVSeznamu();

        /*
        this.actual.setData(null);
        this.actual.setNext(null);
        this.actual.setPrev(null);
         */
        this.actual = null;

        /*
        this.begin.setData(null);
        this.begin.setNext(null);
        this.begin.setPrev(null);
         */
        this.begin = null;

        /*
        this.end.setData(null);
        this.end.setNext(null);
        this.end.setPrev(null);
         */
        this.end = null;
    }

    /**
     * Vloží absolutně první prvek do zatím prázdného ADL a nastaví
     * všechny potřebné ukazatele (na počátek, konec a aaktuální)
     * @param obj datová struktura vkládaných prvků
     */
    private void vlozPrazdny(objType obj) {
        IPrvek<objType> prvek = new Prvek<objType>(obj);

        this.begin = prvek;
        this.end = prvek;
        this.actual = prvek;
        prvek.setData(obj);
        prvek.setNext((Prvek) prvek);
        prvek.setPrev((Prvek) prvek);
    }

    /**
     * Vložení prvku do seznamu na první místo
     *
     * Pokud ještě ADL neobsahuje prvek, vloží první, jinak vloží na první
     * pozici nový prvek a přenastaví potřebné ukazatele.
     *
     * @param obj datová struktura vkládaných prvků
     */
    public void vlozPrvni(objType obj) {
        if (jePrazdny()) {
            this.vlozPrazdny(obj);
        } else {
            IPrvek<objType> prvek = new Prvek<objType>(obj);

            prvek.setNext((Prvek) this.begin);
            prvek.setPrev((Prvek) this.begin.getPrev());
            this.begin.setPrev((Prvek) prvek);
            this.begin = prvek;

            this.end.setNext((Prvek) prvek);
        }
        if (this.pohybAktualnimPriVkladani) {
            this.actual = this.begin;
        }
    }

    /**
     * Vložení prvku do seznamu na poslední místo
     *
     * Pokud ještě ADL neobsahuje prvek, vloží první, jinak vloží na poslední
     * pozici nový prvek a přenastaví potřebné ukazatele.
     *
     * @param obj datová struktura vkládaných prvků
     */
    public void vlozPosledni(objType obj) {
        if (jePrazdny()) {
            this.vlozPrazdny(obj);
        } else {
            IPrvek<objType> prvek = new Prvek<objType>(obj);

            prvek.setNext((Prvek) this.end.getNext());
            prvek.setPrev((Prvek) this.end);
            this.end.setNext((Prvek) prvek);
            this.end = prvek; // změna posledního prvku za nový
            this.begin.setPrev((Prvek) prvek);
        }
        if (this.pohybAktualnimPriVkladani) {
            this.actual = this.end;
        }
    }

    /**
     * Vložení prvku do seznamu jakožto následníka aktuálního prvku
     *
     * Pokud ještě ADL neobsahuje prvek, vloží první, jinak následníka aktuálního
     * prvku a přenastaví potřebné ukazatele. Pokud by byl vkládaný prvek poslední,
     * nastaví nové ukazatele na tento nový poslední prvek.
     *
     * @param obj datová struktura vkládaných prvků
     */
    public void vlozNaslednika(objType obj) {
        if (jePrazdny()) {
            this.vlozPrazdny(obj);
        } else {
            IPrvek<objType> prvek = new Prvek<objType>(obj);
            IPrvek<objType> naslednik = this.actual.getNext();

            prvek.setPrev((Prvek) this.actual);
            this.actual.setNext((Prvek) prvek);

            naslednik.setPrev((Prvek) prvek);
            prvek.setNext((Prvek) naslednik);

//            if(this.actual == this.end) {
//                this.end = prvek;
//            }

            if (naslednik == this.begin) {
                this.end = prvek;
            }
            if (this.pohybAktualnimPriVkladani) {
                this.actual = prvek;
            }
        }
    }

    /**
     * Vložení prvku do seznamu jakožto předchůdce aktuálního prvku
     *
     * Pokud ještě ADL neobsahuje prvek, vloží první, jinak předchůdce aktuálního
     * prvku a přenastaví potřebné ukazatele. Pokud by byl vkládaný prvek první,
     * nastaví nové ukazatele na tento nový první prvek.
     *
     * @param obj datová struktura vkládaných prvků
     */
    public void vlozPredchudce(objType obj) {
        if (jePrazdny()) {
            this.vlozPrazdny(obj);
        } else {
            IPrvek<objType> prvek = new Prvek<objType>(obj);
            IPrvek<objType> predchudce = this.actual.getPrev();

            prvek.setNext((Prvek) this.actual);
            this.actual.setPrev((Prvek) prvek);

            predchudce.setNext((Prvek) prvek);
            prvek.setPrev((Prvek) predchudce);

            if (predchudce == this.end) {
                this.begin = prvek;
            }
            if (this.pohybAktualnimPriVkladani) {
                this.actual = prvek;
            }
        }
    }

    /**
     * Zpřístupnění aktuálního prvku seznamu
     *
     * Pokud ADL zatím neojsahuje prvek, vrátí null, jinak aktuální prvek ADL
     *
     * @return datová struktura vnitřního prvku, nebo null
     */
    public objType zpristupniAktualni() {
        if (jePrazdny()) {
            return null;
        }
        return this.actual.getData();
    }

    /**
     * Zpřístupnění prvního prvku seznamu
     * 
     * Pokud ADL zatím neojsahuje prvek, vrátí null, jinak první prvek ADL
     *
     * @return datová struktura vnitřního prvku, nebo null
     */
    public objType zpristupniPrvni() {
        if (jePrazdny()) {
            return null;
        }
        prvniPrvek();
        return this.begin.getData();
    }

    /**
     * Zpřístupnění posledního prvku seznamu
     *
     * Pokud ADL zatím neojsahuje prvek, vrátí null, jinak poslední prvek ADL
     *
     * @return datová struktura vnitřního prvku, nebo null
     */
    public objType zpristupniPosledni() {
        if (jePrazdny()) {
            return null;
        }
        posledniPrvek();
        return this.end.getData();
    }

    /**
     * Zpřístupnění následníka aktuálního prvku
     *
     * Pokud ADL zatím neojsahuje prvek, vrátí null, jinak následující prvek za
     * aktuálním prvek v ADL
     *
     * @return datová struktura vnitřního prvku, nebo null
     */
    public objType zpristupniNaslednika() {
        if (jePrazdny()) {
            return null;
        }
        dalsiPrvek();
        return this.actual.getData();
    }

    /**
     * Zpřístupnění předchůdce aktuálního prvku
     * 
     * Pokud ADL zatím neojsahuje prvek, vrátí null, jinak předchozí prvek před
     * aktuálním prvkem v ADL
     *
     * @return datová struktura vnitřního prvku, nebo null
     */
    public objType zpristupniPredchudce() {
        if (jePrazdny()) {
            return null;
        }
        predchoziPrvek();
        return this.actual.getData();
    }

    /**
     * Odebrání (vyjmutí) aktuálního prvku ze seznamu aktuální prvek je nastaven
     * na první prvek
     *
     * Do lokální proměnné načte aktuální prvek, zkontroluje na jaké pozici se
     * tento prvek v ADL nachází a podle toho se zachová při odebírání prvku.
     * Pokud se nachází někde mezi prvním a posledním prvkem, přenastaví se
     * ukazatele, jinak se zavolají jiné pšíslušné metody.
     *
     * @return datová struktura vnitřního prvku, nebo null
     */
    public objType odeberAktualni() {
        if (jePrazdny()) {
            return null;
        }
        IPrvek<objType> iPrvekAktualni = this.actual;

        if (jePosledniVSeznamu()) {
            iPrvekAktualni.setData(this.odeberPosledniVSeznamu());
            return iPrvekAktualni.getData();
        }
        if (jePrvni(iPrvekAktualni)) {
            odeberPrvni();
            return iPrvekAktualni.getData();
        }
        if (jePosledni(iPrvekAktualni)) {
            odeberPosledni();
            return iPrvekAktualni.getData();
        }
        {
            iPrvekAktualni.getPrev().setNext((Prvek) iPrvekAktualni.getNext());
            iPrvekAktualni.getNext().setPrev((Prvek) iPrvekAktualni.getPrev());
        }
        iPrvekAktualni.setNext(null);
        iPrvekAktualni.setPrev(null);
        prvniPrvek();
        return iPrvekAktualni.getData();
    }

    /**
     * Odebrání prvního prvku ze seznamu
     *
     * Zkontroluje zda-li není absolutně posledním prvkem v seznamu (zavolá
     * příslušnout funkci) a následně odebere první prvek v seznamu (přenastaví 
     * ukazatele na propojení ADL bez tohoto prvku)
     *
     * @return datová struktura vnitřního prvku, nebo null
     */
    public objType odeberPrvni() {
        IPrvek<objType> iprvek = this.begin;

        if (jePrazdny()) {
            return null;
        }
        if (jePosledniVSeznamu()) {
            iprvek.setData(odeberPosledniVSeznamu());
        } else {
            this.begin = iprvek.getNext();
            this.begin.setPrev((Prvek) this.end);
            this.end.setNext((Prvek) this.begin);
            //this.actual.setData(this.begin.getData());
            if (this.actual == iprvek) {
                prvniPrvek();
            }
        }
        return iprvek.getData();
    }

    /**
     * Odebrání posledního prvku ze seznamu
     *
     * Zkontroluje zda-li není absolutně posledním prvkem v seznamu (zavolá
     * příslušnout funkci) a následně odebere poslední prvek v seznamu (přenastaví
     * ukazatele na propojení ADL bez tohoto prvku)
     *
     * @return datová struktura vnitřního prvku, nebo null
     */
    public objType odeberPosledni() {
        IPrvek<objType> o = this.end;

        if (jePrazdny()) {
            return null;
        }
        if (jePosledniVSeznamu()) {
            o.setData(odeberPosledniVSeznamu());
        } else {
            this.end = o.getPrev();
            this.begin.setPrev((Prvek) end);
            this.end.setNext((Prvek) begin);
            //this.actual.setData(this.end.getData());
            if (this.actual == o) {
                prvniPrvek();
            }
        }
        return o.getData();
    }

    /**
     * Zjistí zda-li je prvek absolutně poslední v seznamu (ukazuje sám na sebe)
     * @return true = je absolutně poslední v seznamu
     */
    private boolean jePosledniVSeznamu() {
        return this.end.getPrev() == this.begin.getPrev()
                && this.begin.getNext() == this.end.getNext();
    }

    /**
     * Odebere absolutně poslední prvek v seznamu
     *
     * Přenastaví všechny ukazatele na aktuální prvek na null
     *
     * @return datová struktura vnitřního prvku
     */
    private objType odeberPosledniVSeznamu() {
        objType data = this.actual.getData();

        this.actual.setData(null);
        this.actual.setNext(null);
        this.actual.setPrev(null);
        this.actual = null;

        this.begin.setNext(null);
        this.begin.setData(null);
        this.begin.setPrev(null);
        this.begin = null;

        this.end.setData(null);
        this.end.setNext(null);
        this.end.setPrev(null);
        this.end = null;

        return data;
    }

    /**
     * Zjistí zda-li je aktuální prvek prvkem prvním v seznamu
     * @return true = je první v seznamu
     */
    private boolean jePrvni() {
        return this.actual == this.begin;
    }

    /**
     * Zjistí zda-li je parametr prvkem prvním v seznamu
     * @param iPrvek Celá vnitřní datová struktura
     * @return true = je první v seznamu (počátek seznamu)
     */
    private boolean jePrvni(IPrvek<objType> iPrvek) {
        return this.begin == iPrvek;
    }

    /**
     * Zjistí zda-li je aktuální prvek prvkem posledním v seznamu
     * @return true = je poslední v seznamu
     */
    private boolean jePosledni() {
        return this.actual == this.end;
    }

    /**
     * Zjistí zda-li je parametr prvkem posledním v seznamu
     * @param iPrvek Celá vnitřní datová struktura
     * @return true = je poslední v seznamu
     */
    private boolean jePosledni(IPrvek<objType> iPrvek) {
        return this.end == iPrvek;
    }

    /**
     * Odebrání následníka aktuálního prvku ze seznamu
     *
     * Zkontroluje pozici v seznamu a podle toho zavolá případnou metodu,
     * pokud se nachází někde uprostřed, přesměruje ukazatele, aby prvek přeskočily.
     *
     * @return datová struktura vnitřního prvku
     */
    public objType odeberNaslednika() {
        IPrvek<objType> o = this.actual.getNext();

        if (jePosledniVSeznamu()) {
            o.setData(this.odeberPosledniVSeznamu());
            return o.getData();
        }
        if (jePrvni(o)) {
            odeberPrvni();
            return o.getData();
        }
        if (jePosledni(o)) {
            odeberPosledni();
            return o.getData();
        }
        {
            o.getPrev().setNext((Prvek) o.getNext());
            o.getNext().setPrev((Prvek) o.getPrev());
        }
        o.setNext(null);
        o.setPrev(null);
        return o.getData();
    }

    /**
     * Odebrání předchůdce aktuálního prvku ze seznamu
     *
     * Zkontroluje pozici v seznamu a podle toho zavolá případnou metodu,
     * pokud se nachází někde uprostřed, přesměruje ukazatele, aby prvek přeskočily.
     *
     * @return datová struktura vnitřního prvku
     */
    public objType odeberPredchudce() {
        IPrvek<objType> o = this.actual.getPrev();

        if (jePosledniVSeznamu()) {
            o.setData(this.odeberPosledniVSeznamu());
            return o.getData();
        }
        if (jePrvni(o)) {
            odeberPrvni();
            return o.getData();
        }
        if (jePosledni(o)) {
            odeberPosledni();
            return o.getData();
        }
        {
            o.getPrev().setNext((Prvek) o.getNext());
            o.getNext().setPrev((Prvek) o.getPrev());
        }
        o.setNext(null);
        o.setPrev(null);
        return o.getData();
    }

    /**
     * Nastaví poslední prvek na aktuální (pohyb seznamem)
     */
    public void posledniPrvek() {
        this.actual = this.end;
    }

    /**
     * Nastaví první prvek na aktuální (pohyb seznamem)
     */
    public void prvniPrvek() {
        this.actual = this.begin;
    }

    /**
     * Posune ukazatel na aktuální na další prvek (pohyb seznamem)
     */
    public void dalsiPrvek() {
        this.actual = this.actual.getNext();
    }

    /**
     * Posune ukazatel na aktuální na předchozí prvek (pohyb seznamem)
     */
    public void predchoziPrvek() {
        this.actual = this.actual.getPrev();
    }

    /**
     * Vytvoří iterátor pro ADL
     * @return ukazatel na vytvořený Iterátor
     */
    public IIterator<objType> vytvorIterator() {
        IIterator adli = new Iterator(TypIteratoru.OD_PREDU);
        return adli;
    }

    public IIterator<objType> vytvorIterator(TypIteratoru typIteratoru) {
        IIterator adli = new Iterator(typIteratoru);
        return adli;
    }

    /*public IIterator<objType> vytvorIterator() {
    IIterator<objType> adli = new Iterator<objType>();
    return adli;
    }*/
    /**
     * Vrátí datovou část aktuálního prvku
     * @return datová struktura vnitřního prvku
     */
    public objType getDataActual() {
        return (objType) this.actual.getData();
    }

    /**
     * Vrátí datovou část prvního prvku
     * @return datová struktura vnitřního prvku
     */
    public objType getDataBegin() {
        return (objType) this.begin.getData();
    }

    /**
     * Vrátí datovou část posledního prvku
     * @return datová struktura vnitřního prvku
     */
    public objType getDataEnd() {
        return (objType) this.end.getData();
    }
}
