/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package Abstrakt.ADL;

import Abstrakt.IIterator;

/**
 *
 * @author Administrator
 */
public interface IAbstrDoubleList<objType> {

    /**
     * Zrušení celého seznamu
     *
     * Postupně prochází celý seznam a ruší jednotlivé prvky, následně
     * zruší i aktuální, začátek a konce seznamu, čímž způsobí,
     * že Garbage collector dealokuje celou ADL
     */
    void Zrus();

    /**
     * Posune ukazatel na aktuální na další prvek (pohyb seznamem)
     */
    void dalsiPrvek();

    /**
     * Vrátí datovou část aktuálního prvku
     * @return datová struktura vnitřního prvku
     */
    objType getDataActual();

    /**
     * Vrátí datovou část prvního prvku
     * @return datová struktura vnitřního prvku
     */
    objType getDataBegin();

    /**
     * Vrátí datovou část posledního prvku
     * @return datová struktura vnitřního prvku
     */
    objType getDataEnd();

    /**
     * Zjistí zda-li je ADL prázdný
     * @return true = je prázdný, false = není prázdný
     */
    boolean jePrazdny();

    /**
     * Odebrání (vyjmutí) aktuálního prvku ze seznamu aktuální prvek je nastaven
     * na první prvek
     *
     * Do lokální proměnné načte aktuální prvek, zkontroluje na jaké pozici se
     * tento prvek v ADL nachází a podle toho se zachová při odebírání prvku.
     * Pokud se nachází někde mezi prvním a posledním prvkem, přenastaví se
     * ukazatele, jinak se zavolají jiné pšíslušné metody.
     *
     * @return datová struktura vnitřního prvku, nebo null
     */
    objType odeberAktualni();

    /**
     * Odebrání následníka aktuálního prvku ze seznamu
     *
     * Zkontroluje pozici v seznamu a podle toho zavolá případnou metodu,
     * pokud se nachází někde uprostřed, přesměruje ukazatele, aby prvek přeskočily.
     *
     * @return datová struktura vnitřního prvku
     */
    objType odeberNaslednika();

    /**
     * Odebrání posledního prvku ze seznamu
     *
     * Zkontroluje zda-li není absolutně posledním prvkem v seznamu (zavolá
     * příslušnout funkci) a následně odebere poslední prvek v seznamu (přenastaví
     * ukazatele na propojení ADL bez tohoto prvku)
     *
     * @return datová struktura vnitřního prvku, nebo null
     */
    objType odeberPosledni();

    /**
     * Odebrání předchůdce aktuálního prvku ze seznamu
     *
     * Zkontroluje pozici v seznamu a podle toho zavolá případnou metodu,
     * pokud se nachází někde uprostřed, přesměruje ukazatele, aby prvek přeskočily.
     *
     * @return datová struktura vnitřního prvku
     */
    objType odeberPredchudce();

    /**
     * Odebrání prvního prvku ze seznamu
     *
     * Zkontroluje zda-li není absolutně posledním prvkem v seznamu (zavolá
     * příslušnout funkci) a následně odebere první prvek v seznamu (přenastaví
     * ukazatele na propojení ADL bez tohoto prvku)
     *
     * @return datová struktura vnitřního prvku, nebo null
     */
    objType odeberPrvni();

    /**
     * Nastaví poslední prvek na aktuální (pohyb seznamem)
     */
    void posledniPrvek();

    /**
     * Posune ukazatel na aktuální na předchozí prvek (pohyb seznamem)
     */
    void predchoziPrvek();

    /**
     * Nastaví první prvek na aktuální (pohyb seznamem)
     */
    void prvniPrvek();

    /**
     * Vložení prvku do seznamu jakožto následníka aktuálního prvku
     *
     * Pokud ještě ADL neobsahuje prvek, vloží první, jinak následníka aktuálního
     * prvku a přenastaví potřebné ukazatele. Pokud by byl vkládaný prvek poslední,
     * nastaví nové ukazatele na tento nový poslední prvek.
     *
     * @param obj datová struktura vkládaných prvků
     */
    void vlozNaslednika(objType obj);

    /**
     * Vložení prvku do seznamu na poslední místo
     *
     * Pokud ještě ADL neobsahuje prvek, vloží první, jinak vloží na poslední
     * pozici nový prvek a přenastaví potřebné ukazatele.
     *
     * @param obj datová struktura vkládaných prvků
     */
    void vlozPosledni(objType obj);

    /**
     * Vložení prvku do seznamu jakožto předchůdce aktuálního prvku
     *
     * Pokud ještě ADL neobsahuje prvek, vloží první, jinak předchůdce aktuálního
     * prvku a přenastaví potřebné ukazatele. Pokud by byl vkládaný prvek první,
     * nastaví nové ukazatele na tento nový první prvek.
     *
     * @param obj datová struktura vkládaných prvků
     */
    void vlozPredchudce(objType obj);

    /**
     * Vložení prvku do seznamu na první místo
     *
     * Pokud ještě ADL neobsahuje prvek, vloží první, jinak vloží na první
     * pozici nový prvek a přenastaví potřebné ukazatele.
     *
     * @param obj datová struktura vkládaných prvků
     */
    void vlozPrvni(objType obj);

    /**
     * Vytvoří iterátor pro ADL
     * @return ukazatel na vytvořený Iterátor
     */
    IIterator<objType> vytvorIterator();

    IIterator<objType> vytvorIterator(TypIteratoru typIteratoru);

    /**
     * Zpřístupnění aktuálního prvku seznamu
     *
     * Pokud ADL zatím neojsahuje prvek, vrátí null, jinak aktuální prvek ADL
     *
     * @return datová struktura vnitřního prvku, nebo null
     */
    objType zpristupniAktualni();

    /**
     * Zpřístupnění následníka aktuálního prvku
     *
     * Pokud ADL zatím neojsahuje prvek, vrátí null, jinak následující prvek za
     * aktuálním prvek v ADL
     *
     * @return datová struktura vnitřního prvku, nebo null
     */
    objType zpristupniNaslednika();

    /**
     * Zpřístupnění posledního prvku seznamu
     *
     * Pokud ADL zatím neojsahuje prvek, vrátí null, jinak poslední prvek ADL
     *
     * @return datová struktura vnitřního prvku, nebo null
     */
    objType zpristupniPosledni();

    /**
     * Zpřístupnění předchůdce aktuálního prvku
     *
     * Pokud ADL zatím neojsahuje prvek, vrátí null, jinak předchozí prvek před
     * aktuálním prvkem v ADL
     *
     * @return datová struktura vnitřního prvku, nebo null
     */
    objType zpristupniPredchudce();

    /**
     * Zpřístupnění prvního prvku seznamu
     *
     * Pokud ADL zatím neojsahuje prvek, vrátí null, jinak první prvek ADL
     *
     * @return datová struktura vnitřního prvku, nebo null
     */
    objType zpristupniPrvni();

}
